<?php

namespace App\Services\channel;

use Api\PhpUtils\Log\FileLog;
use App\Exception\custom\RefundException;
use App\Models\order\mysql\PlatformConfig;
use App\Services\pingxx\PingxxService;
use App\Exception\custom\PayException;
use App\Exception\BaseException;

class Pingxx {

    public function makeOrder($pay_order, $mata_data) {
        if(!empty($pay_order['third_order_id'])) {
            $ret = PingxxService::getInstance()->getOrder($pay_order['third_order_id']);
        } else {
            $ret = PingxxService::getInstance()->createOrder($pay_order, $mata_data);
        }

        if (!empty($ret['error'])) {
            throw new BaseException(['msg'=>$ret['error']['message'], 'code'=>'2301']);
        }

        if ($ret["status"] == 'paid' || $ret['status'] == 'refunded') {
            throw new PayException(['cus' => 101]);

        } elseif ($ret["status"] == 'canceled') {
            throw new PayException(['cus' => 102]);

        } elseif ($ret["status"] == 'created') {
            ;
        } else {
            throw new PayException(['cus' => 103]);
        }
        return $ret;
    }

    /**
     * @param $pay_order
     * @param $mata_data
     * 系统支付
     */
    public function pay($pay_order, $object, $extra = []) {

        $channel_exist = false;
        $channel = $this->pay2Channel($pay_order['pay_channel']);
        if(!empty($object['charges']['data'])) {
            //是否存在
            foreach ($object['charges']['data'] as $ch) {
                if($ch['channel'] == $channel) {
                    $channel_exist = true;
                }
            }
        }

        //判断是否做过pay操作 charge
        if($channel_exist == false) {
            // 创建支付单
            $payment = [
                'charge_order_no'=> $pay_order['pay_order_id'],
                'charge_amount'=> $pay_order['pay_amount'],
                'channel'=> $channel,
                'extra'=> $extra,
            ];

            $pay = PingxxService::getInstance()->pay($object['id'], $payment);
            if (!empty($pay['error'])) {
                throw new BaseException(['msg'=>$pay['error']['message'], 'code'=>'2002']);
            }

            return $pay;

        } else {
            return $object;
        }
    }

    /**
     * @param $pay_order
     * @param $mata_data
     * 系统退款
     */
    public function refund($data, $mata_data) {

        /*
        https://www.pingxx.com/api/%E8%AE%A2%E5%8D%95%20Refunds%20%E9%80%80%E6%AC%BE%E6%A6%82%E8%BF%B0.html
        订单退款 paid = true, status = canceled, amount_paid > 0 则表明可以退款
        amount_paid 大于 actual_amount 需要最后一单全退。
        具体见ping++ 文档
        */

        $pay = PingxxService::getInstance()->getOrder($data['pay_order']['third_order_id']);
        if(empty($pay['paid']) || $pay['paid'] != true) {
            throw new RefundException(['cus' => 7]);
        }

        // 存在多次支付的场景需要人工介入处理， 修复订单支付金额，退款金额
        if($pay['amount_paid'] > $pay['actual_amount']) {
            FileLog::error('pay-service: 订单可能存在多次成功支付，需人工确认处理。', json_encode($pay, JSON_UNESCAPED_UNICODE));
            throw new RefundException(['cus' => 8]);
        }

        // 构造退款数据
        $refund = [
            'id'=>$pay['id'], //pingxx订单ID
            'charge'=>$pay['charge'], //pingxx 支付订单ID
            'refund_mode' => 'to_source', //默认返回
            'charge_amount'=>$data['refund_order']['refund_amount'],
            'metadata' => $mata_data,
        ];

        return PingxxService::getInstance()->sendRefund($refund);
    }

    private function pay2Channel($pay_method_id) {

        $maps = [
            1=>"syt",	//系统余额支付
            105=>"wx",	//微信 App 支付
            106=>"alipay",	//支付宝
            107=>"wx_pub",	//微信 JSAPI 支付
            108=>"wx_pub_qr",	//微信 Native 支付
            109=>"wx_wap",	//微信 H5 支付
            110=>"wx_lite",	//微信小程序支付
        ];

        if(!empty($maps[$pay_method_id])) {
            return $maps[$pay_method_id];
        } else {
            return $maps[105];
        }
    }

    /**
     * @param $pay_order
     * @return array
     * @throws PayException
     * 手续费计算
     */
    public function clearChannelTips($pay_order) {

        //PlatformConfig
        $info = PlatformConfig::get('*', [
            'account_type'=>2,
            'channel'=>1,
            'source_name'=>$pay_order['source_name'],
            'service_name'=>$pay_order['service_name']
        ]);

        if(empty($info)) {
            throw new PayException(['cus'=>104]);
        }

        $float_tip = $info['rate'] * $pay_order['pay_amount'] / 10000;
        if($float_tip >= 1 ) {
            $total_tip = round($float_tip);

            return [
                'account_id' => $info['account_id'],
                'amount' => $total_tip,
            ];

        } else {
            return [];
        }
    }

    public function clearPlatformTips($pay_order) {
        //PlatformConfig
        $info = PlatformConfig::get('*', [
            'account_type'=>1,
            'channel'=>1,
            'source_name'=>$pay_order['source_name'],
            'service_name'=>$pay_order['service_name']
        ]);

        if(empty($info)) {
            throw new PayException(['cus'=>104]);
        }

        $float_tip = $info['rate'] * $pay_order['total_price'] / 10000;
        $total_tip = round($float_tip);
        return [
            'account_id' => $info['account_id'],
            'amount' => $total_tip,
        ];
    }

    public function getFundsInfo($pay_order) {
        $info = PlatformConfig::get('*', [
            'account_type'=>3,
            'channel'=>1,
            'source_name'=>$pay_order['source_name'],
            'service_name'=>$pay_order['service_name']
        ]);

        if(empty($info)) {
            throw new PayException(['cus'=>104]);
        }

        return $info;
    }

}
