<?php


namespace App\Services\marketing;


use App\Models\order\mysql\MarketingAccount;
use App\Models\order\mysql\MarketingAccountLog;
use App\Models\order\mysql\MarketingAccountMap;
use Api\PhpServices\Idgen\Idgen;
use App\Exception\custom\AccountException;

class AccountService
{
    private static $maps = [];
    /**
     * @param $list
     * @param $order_id
     * @return bool
     * 资金冻结
     */
    public static function frozen($list, $order_id) {

        $coupon = self::formatData($list);

        $cnt = true;

        foreach ($coupon as $id=>$amount) {

            $marketing_account_id = self::map($id);
            $cnt = $cnt && MarketingAccount::update([
                'frozen_amount[+]'=>$amount,
                'receivable_amount[-]'=>$amount,
            ], ['marketing_account_id'=>$marketing_account_id]);

            $cnt = $cnt && MarketingAccountLog::insert([
                'marketing_account_id'=>$marketing_account_id,
                'third_id'=>$id,
                'fund_type'=>1,
                'trade_id'=>$order_id,
                'amount'=>$amount,
            ], ['rowCount'=>true]);
        }

        return $cnt;
    }

    /**
     * @param $list
     * @param $order_id
     * @return bool
     * 营销资金扣款
     */
    public static function pay($list, $order_id) {

        $coupon = self::formatData($list);

        $cnt = true;
        // 冻结优惠信息
        foreach ($coupon as $id=>$amount) {
            $marketing_account_id = self::map($id);
            $cnt = $cnt && MarketingAccount::update([
                'frozen_amount[-]'=>$amount,
            ], ['marketing_account_id'=>$marketing_account_id]);

            $cnt = $cnt && MarketingAccountLog::insert([
                'marketing_account_id'=>$marketing_account_id,
                'third_id'=>$id,
                'fund_type'=>2,
                'trade_id'=>$order_id,
                'amount'=>$amount,
            ], ['rowCount'=>true]);
        }

        return $cnt;
    }

    /**
     * @param $list
     * @param $refund_order_id
     * @return bool
     * 退款营销可用增加
     */
    public static function refund($list, $refund_order_id) {

        $coupon = self::formatData($list);
        $cnt = true;

        foreach ($coupon as $id=>$amount) {
            $marketing_account_id = self::map($id);
            $cnt = $cnt && MarketingAccount::update([
                    'receivable_amount[+]'=>$amount,
                ], ['marketing_account_id'=>$marketing_account_id]);

            $cnt = $cnt && MarketingAccountLog::insert([
                    'marketing_account_id'=>$marketing_account_id,
                    'third_id'=>$id,
                    'fund_type'=>3,
                    'trade_id'=>$refund_order_id,
                    'amount'=>$amount,
                ], ['rowCount'=>true]);
        }
        return $cnt;
    }

    /**
     * @param $list
     * @param $order_id
     * @return bool
     * 未支付取消，营销冻结释放
     */
    public static function cancel($list, $order_id) {
        $coupon = self::formatData($list);
        $cnt = true;

        foreach ($coupon as $id=>$amount) {
            $marketing_account_id = self::map($id);
            $cnt = $cnt && MarketingAccount::update([
                    'frozen_amount[-]'  => $amount,
                    'receivable_amount[+]'=>$amount,
                ], ['marketing_account_id'=>$marketing_account_id]);

            $cnt = $cnt && MarketingAccountLog::insert([
                    'marketing_account_id'=>$marketing_account_id,
                    'third_id'=>$id,
                    'fund_type'=>4,
                    'trade_id'=>$order_id,
                    'amount'=>$amount,
                ], ['rowCount'=>true]);
        }
        return $cnt;
    }

    /**
     * @param $data
     * @param $marketing_account_id
     * @return bool
     * 营销充值
     */
    public static function recharge($data, $marketing_account_id) {

        $cnt = true;
        $cnt = $cnt && MarketingAccount::update([
                'receivable_amount[+]'=>$data['amount']
            ], ['marketing_account_id'=>$marketing_account_id]);

        $cnt = $cnt && MarketingAccountLog::insert([
            'marketing_account_id'=>$marketing_account_id,
            'third_id'=>$data['third_id'],
            'fund_type'=>5,
            'trade_id'=>$data['trade_id'],
            'amount'=>$data['amount'],
        ]);

        return $cnt;
    }

    public static function transfer($data, $marketing_account_id) {

        $cnt = true;
        $cnt = $cnt && MarketingAccount::update([
                'receivable_amount[-]'=>$data['amount']
            ], ['marketing_account_id'=>$marketing_account_id]);
        $cnt = $cnt && MarketingAccountLog::insert([
            'marketing_account_id'=>$marketing_account_id,
            'third_id'=>$data['third_id'],
            'fund_type'=>6,
            'trade_id'=>$data['trade_id'],
            'amount'=>$data['amount'],
        ]);

        return $cnt;
    }

    public static function map($third_id, $source_name = 10, $service_name = 1) {

        if(!isset(self::$maps[$third_id])) {
            $info = MarketingAccountMap::get('*',
                ['third_id'=>$third_id, 'source_name'=>$source_name, 'service_name'=>$service_name]);
            if($info) {
                self::$maps[$third_id] = $info['marketing_account_id'];
            } else {
                self::$maps[$third_id] = '';
            }
        }

        return self::$maps[$third_id];
    }

    private static function formatData($list) {

        $coupon = [];
        foreach ($list as $r) {
            if(!isset($coupon[$r['capital_pool_id']])) {
                $coupon[$r['capital_pool_id']] = 0;
            }
            $coupon[$r['capital_pool_id']] += $r['share_coupon_amount'];
        }

        return $coupon;
    }

    public static function get_idgen_id($count=1){
        $res = Idgen::get(appConfig('idgen.partner'),appConfig('idgen.key'), [[
            "type" => "pay_marketing_account_id",
            "count"=> $count]]);
        if(!$res['id_snow']['pay_marketing_account_id']){
            throw new AccountException(['cus' => 0]);
        }
        return $res['id_snow']['pay_marketing_account_id'][0];
    }
}