<?php


namespace App\Services\pay;

use Api\PhpServices\Idgen\Idgen;
use Api\PhpUtils\Log\FileLog;
use App\Models\order\mysql\PayOrder;
use App\Models\order\mysql\PayOrderItem;
use App\Models\order\mysql\PayCouponItem;
use App\Models\order\mysql\PayOrderClearing;
use App\Models\order\mysql\PayOrderClearingItem;
use App\Models\order\mysql\WriteOffMerchantRecord;
use App\Models\order\mysql\WriteOffOrder;
use App\Models\order\mysql\WriteOffOrderItem;
use App\Models\order\mysql\WriteOffPersonalRecord;
use App\Models\order\mysql\WriteOffPlatformRecord;
use App\Services\Channel;
use App\Services\marketing\AccountService;
use App\Services\order\OrderService;

use App\Services\pingxx\PingxxService;
use App\Services\wallet\MerchantAccountService;
use App\Services\wallet\WalletService;
use Helpers\Strategy;
use Helpers\Sdk;
use Api\PhpServices\Wechat\Msg;
use Yaf\Application;

use App\Exception\custom\PayException;
use App\Exception\BaseException;

class PayService
{
    // 订单信息
    private $order_info = [];
    // 子订单信息
    private $order_items = [];

    private $pay_order = [];
    private $clear_list = [];
    private $clear_items_list = [];
    private $gen_ids = [];


    /**
     * @param $params
     * @return mixed
     * @throws PayException
     * @throws \App\Exception\custom\CodeSpecialException
     * 支付订单校验
     */
    private function pay_validate($params) {

        if(empty($params['order_id'])) {
            throw new PayException(['cus' => 1]);
        }

        if(empty($params['user_id'])) {
            throw new PayException(['cus' => 2]);
        }

        // 获取订单 + 子单信息
        $data = OrderService::getFullOrderData($params['order_id']);
        if (empty($data['result']['order_info']) || empty($data['result']['order_item_list'])) {
            throw new PayException(['cus' => 3]);
        }

        $order_info = $data['result']['order_info'];
        if ($order_info['user_id'] != $params['user_id']) {
            throw new PayException(['cus' => 4]);
        }

        if (empty($order_info['pay_expiration_time'])) {
            throw new PayException(['cus' => 5]);
        }

        if(strtotime($order_info['pay_expiration_time']) < time()) {
            throw new PayException(['cus' => 6]);
        }

        return $data['result'];
    }

    /**
     * @param $data
     * @param $pay_method_id
     * @return array
     * @throws PayException
     * 创建/获取支付订单信息
     */
    private function make_pay_order($data, $pay_method_id) {

        $order = $data["order_info"];
        $pay_expiration_time = strtotime($order['pay_expiration_time']);
        $now = time();

        if($pay_expiration_time - $now < 70 ) {
            $pay_expiration_time = $now + 70;
        }

        $ids = $this->get_idgen_id($order["user_id"], 1 + count($data['order_item_list']));
        $pay_order_id = array_shift($ids);

        PayOrder::beginTransaction();
        $pay = PayOrder::getMaster('*', ['order_id'=>$order['order_id']]);
        if(!empty($pay)) {
            //更新支付渠道
            if($pay['pay_order_status'] == 0) {
                PayOrder::update(['pay_channel'=>$pay_method_id],
                    ['pay_order_id'=>$pay['pay_order_id']]
                );
            }
            PayOrder::commit();
            return $pay;
        }

        // 新增支付订单
        $new_pay = [
            'pay_order_id' => $pay_order_id,
            'user_id' => $order['user_id'],
            'life_account_id' => $order['life_account_id'],
            'shop_id' => $order['shop_id'],
            'order_id' => $order['order_id'],
            'pay_order_status' => 0,
            'total_price' => $order['total_price'],
            'pay_discount' => $order['discount_fee'], //pay_discount
            'pay_amount' => $order['payment'],
            'pay_channel' => $pay_method_id,
            'third_order_id' => '',
            'expire_time' => date('Y-m-d H:i:s', $pay_expiration_time),
            'source_name' => 10,
            'service_name' => 1,
            'business_from' => $order['business_from'],
            'extra' => json_encode([]),
        ];

        $cnt = true;
        $cnt = $cnt && PayOrder::insert($new_pay, ['rowCount'=>true]);

        $order_items = $coupon_items = [];
        foreach ($data['order_item_list'] as $r) {
            $order_items[] = [
                'pay_order_item_id'=>array_shift($ids),
                'pay_order_id'=>$pay_order_id,
                'order_id'=>$r['order_id'],
                'order_item_id'=>$r['order_item_id'],
                'user_id'=>$r['user_id'],
                'goods_spu_id'=>$r['goods_spu_id'],
                'goods_sku_id'=>$r['goods_sku_id'],
                'goods_version'=>$r['goods_version'],
                'goods_name'=>$r['goods_name'],
                'goods_type'=>$r['goods_type'],
                'goods_sale_type'=>$r['goods_sale_type'],
                'rule_refund'=>$r['rule_refund'],
                'price'=>$r['price'],
                'goods_num'=>$r['goods_num'],
                'payment'=>$r['payment'],
                'share_coupon'=>$r['share_coupon'],
            ];
        }

        //新增订单子单
        $cnt = $cnt && PayOrderItem::insert($order_items, ['rowCount'=>true]);

        if(!empty($data['order_share_coupon'])) {
            foreach ($data['order_share_coupon'] as $r) {
                $coupon_items[] = [
                    'pay_order_id'=>$pay_order_id,
                    'order_id'=>$r['order_id'],
                    'user_id'=>$r['user_id'],
                    'order_item_id'=>$r['order_item_id'],
                    'coupon_id'=>$r['coupon_id'],
                    'capital_pool_id'=>$r['capital_pool_id'],
                    'coupon_amount'=>$r['coupon_amount'],
                    'share_coupon_amount'=>$r['share_coupon_amount'],
                ];
            }

            //新增订单优惠信息
            $cnt = $cnt && PayCouponItem::insert($coupon_items, ['rowCount'=>true]);

            //营销资金冻结
            $cnt = $cnt && AccountService::frozen($data['order_share_coupon'], $order['order_id']);
        }

        if($cnt == true) {
            $pay = $new_pay;
            PayOrder::commit();
        } else {
            PayOrder::rollback();
            throw new PayException(['cus' => 7]);
        }

        return $pay;
    }

    /**
     * @param $params
     * @param array $extra
     * @return array|mixed
     * @throws PayException
     * 创建支付订单
     */
    public function do_pay($params, $extra = [])
    {
        // 支付数据检查
        $data = $this->pay_validate($params);

        // 创建支付订单、营销冻结
        $pay_order = $this->make_pay_order($data, $params['pay_method_id']);
        if ($pay_order['pay_order_status'] == 2) {
            throw new PayException(['cus' => 8]);
        }

        // 支付金额0，系统支付
        if($pay_order['pay_amount'] == 0) {
            $params['pay_method_id'] = 1;
        }

        // 构造支付回调的 mataData
        $meta_data = [
            'app_id'=>'merchant-c',
            'environ'=>Application::app()->environ(),
            'order_id'=>$params["order_id"],
            'goods'=>[],
            'marketing'=>[],
            'order_item_id'=>[],
        ];

        foreach ($data['order_item_list'] as $item) {
            if(empty($meta_data['goods'][$item['goods_sku_id']])) {
                $meta_data['goods'][$item['goods_sku_id']] = 0;
            }
            $meta_data['goods'][$item['goods_sku_id']] += $item['goods_num'];

            if(!empty($item['marketing_id'])) {
                $meta_data['marketing'][$item['goods_sku_id']] = $item['marketing_id'];
            }

            array_push($meta_data['order_item_id'], $item['order_item_id']);
        }

        try{
            $payer = Channel::getChannel($params['pay_method_id']);
            $ret = $payer->makeOrder($pay_order, $meta_data);

            PayOrder::update(['pay_order_status'=>1, 'third_order_id'=>$ret['id']],
                ['pay_order_id'=>$pay_order['pay_order_id'], 'pay_order_status'=>0]);

            $response = $payer->pay($pay_order, $ret, $extra);

        }catch (BaseException $e) {
            throw $e;

        } catch (\Exception $e) {
            throw new BaseException(['msg'=>$e->getMessage(), 'code'=>'2301']);
        }

        return $response;
    }

    /**
     * @param $data
     * @return array|string[]
     * @throws PayException
     * 回调订单支付
     */
    public function call_back($data)
    {
        if (!in_array($data['type'], ['charge.succeeded'])) {
            return ['error'=>'支付回调类型不正确'];
        }

        if (empty($data['data']['object'])) {
            return ['error'=>'支付回调格式不正确'];
        }

        $object = $data['data']['object'];
        if(empty($object['metadata']['app_id'])
            || substr($object['metadata']['app_id'], 0, 9) != "merchant-") {
            //非支付中心发出的订单，需要忽略
            return ['error'=>'非merchant渠道订单，不做处理。'];
        }

        if($object['paid'] != true) {
            return ['error'=>'支付状态不正确，不做处理。'];
        }

        $pay_order_id = $object['order_no'];
        $pay_order = PayOrder::get('*', ['pay_order_id' => $pay_order_id]);
        if (empty($pay_order)) {
            throw new PayException(['cus' => 7]);
        }
        if($pay_order['pay_amount'] != $object['amount']) {
            throw new PayException(['cus' => 8]);
        }

        // 修改状态，锁定业务
        $cnt = PayOrder::update([
            'pay_order_status' => 2, //已经支付
            'pingxx_callback_success_time' => date('Y-m-d H:i:s'),
        ], [
            'pay_order_id' => $pay_order_id,
            'pay_order_status' => 1, //待支付确认
        ]);

        //保持幂等返回成功
        if($cnt == 0) {
            return ['pay_order_id' => $pay_order_id];
        }

        $this->pay_order = $pay_order;
        $payer = Channel::getChannel($pay_order['pay_channel']);

        // 生成支付清分单
        $this->make_order_clearing($payer);

        try{
            //收集第一商品名称
            if($this->order_items) {
                $title = $this->order_items[0]['goods_name'] ?? '商品名称';
            } else {
                $title = '';
            }

            //通知消息
            $app_id = 'wx65e9ba99c333444f';
            $template_id = 'EzOH2ZpfW-eUdFf9gzjZTAv-Z9PFmKHNsQ0VOGvvN7c';
            $page_url = \Yaf\Application::app()->getConfig()->get('wxApp.jw.order');
            $type = 1;
            $query = [
                $title,
                $pay_order['order_id'],
                round($pay_order['pay_amount'] / 100, 2),
                date('Y-m-d'),
            ];

            if($pay_order['business_from'] != 2) { //接龙业务不发送消息
                Msg::send($pay_order['user_id'], $app_id, $template_id, $page_url, $type, $query);
            }

        }catch (\Throwable $e) {
            ;
        }

        return ['pay_order_id' => $pay_order_id];
    }

    /**
     * @param $order_id
     * @return array
     * @throws BaseException
     * @throws PayException
     * 返回订单状态及订单渠道列表
     */
    public function pay_charges($order_id) {

        $pay_order = PayOrder::get('*', ['order_id'=>$order_id]);
        if(empty($pay_order)) {
            throw new PayException(['cus' => 18]);
        }

        $data = [
            'status'=>'created',
            'max_expire_time'=>$pay_order['expire_time'],
            'list'=>[]
        ];

        $max_expire_time = strtotime($pay_order['expire_time']);

        if($max_expire_time < time()) {
            $data["status"] = 'canceled';
        }

        if(empty($pay_order['third_order_id'])) {
            return $data;
        }

        //paid、refunded、canceled、created
        $ret = PingxxService::getInstance()->getOrder($pay_order['third_order_id']);
        if (!empty($ret['error'])) {
            throw new BaseException(['msg'=>$ret['error']['message'], 'code'=>'2301']);
        }

        $data["status"] = $ret["status"];
        if(!empty($ret['charges']['data'])) {
            //是否存在
            foreach ($ret['charges']['data'] as $ch) {

                $data["list"][] = [
                    'paid'=>$ch['paid'],
                    'refunded'=>$ch['refunded'],
                    'channel'=>$ch['channel'],
                    'time_expire'=>$ch['time_expire'],
                ];

                if($max_expire_time < $ch['time_expire']) {
                    $max_expire_time = $ch['time_expire'];
                }
            }
        }

        $data['max_expire_time'] = date('Y-m-d H:i:s', $max_expire_time);
        return $data;
    }

    /**
     * @param $order_id
     * @param $life_account_id
     * @param array $order_item_ids
     * @return array
     * @throws BaseException
     * @throws \App\Exception\custom\CodeSpecialException
     */
    public function write_off($order_id, $life_account_id, $order_item_ids = [] )
    {
        try{
            PayOrderItem::beginTransaction();
            $pay_order = PayOrder::getMaster('*', ['order_id'=>$order_id]);
            if(empty($pay_order) || $pay_order['life_account_id'] != $life_account_id) {
                throw new PayException(['cus'=>14]);
            }

            $list = PayOrderItem::getMaster('*', ['pay_order_id' => $pay_order['pay_order_id']]);
            if(empty($list)) {
                throw new PayException(['cus'=>15]);
            }

            $item_ids = [];
            foreach ($list as $item) {
                if(in_array($item['refund_order_status'], [1, 2])) {
                    continue; //退款中、已退款不允许再次申请
                }
                if($item['can_notify_account'] == 1) {
                    continue; //已结算不允许再次核销
                }
                if(!empty($order_item_ids) && !in_array($item['order_item_id'], $order_item_ids)) {
                    continue; //部分核销，且不再核销子单列表，不处理
                }

                $item_ids[] = $item['order_item_id'];
            }

            if(empty($item_ids)) {
                throw new PayException(['cus'=>16]);
            }

            $ids = $this->gen_pay_order_id($life_account_id, count($list) + 5);
            $write_off_order_id = array_shift($ids);

            //统计平台、团长、商户金额
            $cnt = true;
            $cnt = $cnt && WriteOffOrder::insert([
                'write_off_order_id'=>$write_off_order_id,
                'user_id'=>$pay_order['user_id'],
                'order_id'=>$pay_order['order_id'],
                'write_off_status'=>0,
                ], ['rowCount' => true]);

            $items = [];
            foreach ($item_ids as $_id) {
                $items[] = [
                    'writeoff_order_item_id'=>array_shift($ids),
                    'writeoff_order_id'=>$write_off_order_id,
                    'order_item_id'=>$_id,
                ];
            }

            $cnt = $cnt && WriteOffOrderItem::insert($items, ['rowCount' => true]);

            $cnt = $cnt && $this->write_off_clear_record($write_off_order_id, $pay_order, $item_ids);
            $cnt = $cnt && PayOrderItem::update([
                'notify_account_status' => 0,
                'can_notify_account' => 1,
                'can_notify_account_time' => date('Y-m-d H:i:s'),
            ], ['order_item_id' => $item_ids, 'can_notify_account'=>0, 'refund_order_status'=>0]);

            if($cnt == false) {
                throw new PayException(['cus'=>17]);
            }
            PayOrderItem::commit();
        }catch (PayException $e) {
            PayOrderItem::rollback();
            throw $e;

        }catch (\Exception $e) {
            PayOrderItem::rollback();
            throw new BaseException(['msg'=>$e->getMessage(), 'code'=>$e->getCode()]);
        }

        $this->sync_account_wallet($write_off_order_id);

        return [
            'write_off_order_id'=>$write_off_order_id,
            'order_id'=>$order_id,
            'order_item_id' => $item_ids
        ];
    }

    /**
     * @param $write_off_order_id
     * @return bool
     */
    public function sync_account_wallet($writeOff_order_id) {

        $p_cnt = $u_cnt = $m_cnt = true;

        //修改状态为执行中
        $cnt = WriteOffOrder::update([
            'notify_account_status'=>1,
        ], ['write_off_order_id'=>$writeOff_order_id, 'write_off_status' => 2, 'notify_account_status'=>0]);

        if(!$cnt) {//乐观锁，不满足条件不处理
            return true;
        }

        //平台收入入账
        $p = WriteOffPlatformRecord::selectMaster('*',
            ['writeoff_order_id'=>$writeOff_order_id, 'notify_account_status[<]'=>2]);
        if($p) {
            $p_cnt = $p_cnt && PlatformAccountService::credit(['trade_id'=>$writeOff_order_id, 'amount'=>$p['payment']]);
            $p_cnt = $p_cnt && WriteOffPlatformRecord::update([
                'notify_account_status'=>2,
                'notify_account_success_time'=>date('Y-m-d H:i:s'),
            ], ['writeoff_order_id'=>$writeOff_order_id]);
        }

        //团长分销入账
        $us = WriteOffPersonalRecord::getMaster('*',
            ['writeoff_order_id'=>$writeOff_order_id, 'notify_account_status[<]'=>2]);
        if($us) {
            $w_list = [];
            foreach ($us as $u) {
                $w_list[] = [
                    'user_id' => $u['account_id'],
                    'service_name' => 10,
                    'source_name' => 1,
                    'amount' => $u['payment'],
                    'third_order_id' => $u['writeoff_personal_id'],
                    'third_order_id_type' => 406,
                    'third_order_desc' => '',
                ];
            }

            try{
                WalletService::send(['list'=>$w_list]);
                $u_cnt = $u_cnt && WriteOffPlatformRecord::update([
                        'notify_account_status'=>2,
                        'notify_account_success_time'=>date('Y-m-d H:i:s'),
                    ], ['writeoff_order_id'=>$writeOff_order_id]);

            }catch (\Exception $e) {
                $u_cnt = false;
            }
        }

        //商家收益 407
        $m = WriteOffMerchantRecord::selectMaster('*',
            ['writeoff_order_id'=>$writeOff_order_id, 'notify_account_status[<]'=>2]);
        if($m) {
            try{ //MerchantAccountService
                MerchantAccountService::sync([
                    'life_account_id'=>$m['life_account_id'],
                    'shop_id'=>$m['shop_id'],
                    'source_name'=>$m['source_name'],
                    'service_name'=>$m['service_name'],
                    'write_off_order_id'=>$m['write_off_order_id'],
                    'third_order_title'=>$m['title'],
                    'third_order_desc'=>$m['desc'],
                    'third_order_type'=>$m['business_from'],
                    'amount'=>$m['payment'],
                ]);

                $m_cnt = $m_cnt && WriteOffPlatformRecord::update([
                        'notify_account_status'=>2,
                        'notify_account_success_time'=>date('Y-m-d H:i:s'),
                    ], ['writeoff_order_id'=>$writeOff_order_id]);

            }catch (\Exception $e) {

                $m_cnt = false;
            }
        }

        if($m_cnt && $u_cnt && $p_cnt) {
            WriteOffOrder::update([
                'notify_account_status'=>2,
                'notify_account_success_time'=>date('Y-m-d H:i:s'),
            ], ['writeoff_order_id'=>$writeOff_order_id, 'notify_account_status'=>1]);

        } else {
            $info = WriteOffOrder::selectMaster('*', ['writeoff_order_id'=>$writeOff_order_id]);
            $next = $this->getNextTime($info['can_notify_account_time']);
            WriteOffOrder::update([
                'notify_account_status'=>0,
                'can_notify_account_time[+]'=>1,
                'notify_account_times'=>date('Y-m-d H:i:s', $next),
            ], ['writeoff_order_id'=>$writeOff_order_id]);
        }
    }

    /**
     * @param $writeOff_order_id
     * @param array $order_item_ids
     * @return bool
     */
    private function write_off_clear_record($writeOff_order_id, $pay_order, $order_item_ids = []) {

        $clear_items = PayOrderClearingItem::selectMaster('*',
            ['order_item_id'=>$order_item_ids]);

        $d = $m = $p = [];
        foreach ($clear_items as $r) {
            if($r['pay_sub_type'] == 404) { // 团长佣金收入
                if(!isset($d[$r['account_id']])) {
                    $d[$r['account_id']] = [
                        'writeoff_merchant_id'=>array_shift($this->gen_ids),
                        'writeOff_order_id'=>$writeOff_order_id,
                        'pay_order_id'=>$r['pay_order_id'],
                        'order_id'=>$r['order_id'],
                        'user_id'=>$r['account_id'],
                        'payment'=>0,
                    ];
                }
                $d[$r['account_id']]['payment'] += $r['pay_amount'];

            } elseif ($r['pay_sub_type'] == 405 || $r['pay_sub_type'] == 408) { // 平台抽成
                if(!isset($p[$r['account_id']])) {
                    $p[$r['account_id']] = [
                        'refund_platform_id'=>array_shift($this->gen_ids),
                        'refund_order_id'=>$writeOff_order_id,
                        'pay_order_id'=>$r['pay_order_id'],
                        'order_id'=>$r['order_id'],
                        'account_id'=>$r['account_id'],
                        'payment'=>0,
                    ];
                }
                $p[$r['account_id']]['payment'] += $r['pay_amount'];

            } elseif ($r['pay_sub_type'] == 407) { // 商品交易
                $key = $r['life_account_id'] . '_' . $r['shop_id'];
                if(!isset($m[$key])) {
                    $m[$key] = [
                        'writeoff_merchant_id'=>array_shift($this->gen_ids),
                        'writeoff_order_id'=>$writeOff_order_id,
                        'pay_order_id'=>$r['pay_order_id'],
                        'order_id'=>$r['order_id'],
                        'life_account_id'=>$pay_order['life_account_id'],
                        'shop_id'=>$pay_order['shop_id'],
                        'source_name'=>$pay_order['source_name'],
                        'service_name'=>$pay_order['service_name'],
                        'business_from'=>$pay_order['business_from'],
                        'title'=>'',
                        'desc'=>'',
                        'payment'=>0,
                    ];
                }
                $pre = ',';
                if(empty($m[$key]['title'])) {
                    $m[$key]['title'] = $r['remark'];
                    $pre = '';
                }
                $m[$key]['desc'] .= $pre . $r['remark'];
                $m[$key]['payment'] += $r['pay_amount'];

            } else {}
        }

        $cnt = true;
        $cnt = $cnt && WriteOffMerchantRecord::insert(array_values($m), ['rowCount' => true]);

        if($p) {
            $cnt = $cnt && WriteOffPlatformRecord::insert(array_values($p), ['rowCount' => true]);
        }
        if($d) {
            $cnt = $cnt && WriteOffPersonalRecord::insert(array_values($d), ['rowCount' => true]);
        }

        return $cnt;
    }

    /**
     * @param $payer
     * @throws PayException
     * 负责支付的清分
     */
    private function make_order_clearing($payer)
    {
        /**
         * 1. 总账入账，补贴扣减
         * 2. 手续费、团长、平台、商户
         * 3. 用户支付 + 营销补贴 = 手续费 + 团长佣金 + 平台抽成 + 商户收入
         */

        $this->clear_list = [];
        $this->clear_items_list = [];
        $this->order_items = PayOrderItem::selectMaster('*',
            ['pay_order_id'=>$this->pay_order['pay_order_id']]);

        $coupon_items = PayCouponItem::selectMaster('*', ['pay_order_id'=>$this->pay_order['pay_order_id']]);

        //生成IDS
        $this->get_idgen_id($this->pay_order['user_id'], 3 + count($this->order_items) * 6);

        $wx_tip = $this->make_tip_clearing($payer); //渠道手续费
        $distribution_tip = $this->make_distribution_clearing(); //团长收益
        $platform = $this->make_platform_clearing($payer); //平台收益

        $merchant_cash = $this->pay_order['pay_amount'] + $this->pay_order['pay_discount'] - $wx_tip - $distribution_tip - $platform['amount'];

        //核对各项资金的合理性
        if($wx_tip <0 || $distribution_tip <0 || $platform['amount'] < 0 || $merchant_cash < 0) {
            FileLog::error('pay-service: 支付预结算失败，结算资金为负',
                json_encode([
                    'funds'=>$this->pay_order['payment'],
                    'wx_tip'=>$wx_tip,
                    'marketing_tip'=>$this->pay_order['pay_discount'],
                    'distribution_tip'=>$distribution_tip,
                    'platform_tip'=>$platform['amount'],
                    'merchant_cash'=>$merchant_cash,
                ]));
            throw new PayException(['cus'=>12]);
        }

        $this->make_merchant_clearing($merchant_cash, $platform); //商户收益

        try {
            $cnt = true;
            PayOrderClearing::beginTransaction();

            //总账资金记账
            $cnt = $cnt && FundsService::credit([
                'trade_id'=>$this->pay_order['pay_order_id'],
                'third_order_id'=>$this->pay_order['third_order_id'],
                'amount'=>$this->pay_order['pay_amount'],
                'tip'=>$wx_tip,
            ], 1);

            //营销扣款
            $cnt = $cnt && AccountService::pay($coupon_items, $this->pay_order['order_id']);
            $cnt = $cnt && PayOrderClearing::insert($this->clear_list, ['rowCount' => true]);
            $cnt = $cnt && PayOrderClearingItem::insert($this->clear_items_list, ['rowCount' => true]);
            if ($cnt == false) {
                throw new PayException(['cus' => 0]);
            }
            PayOrderClearing::commit();
        } catch (\Exception $e) {
            PayOrderClearing::rollback();
            throw $e;
        }

        //如果是不可退款商品，则需进入结算队列
        foreach ($this->order_items as $r) {
            // 收集直接到账信息
            //rule_refund 退款规则：1到期自动退款，2购买后不允许退款，3到期后不退款
            if($r['rule_refund'] == 2 ) { //分销商品一定是不可退
                PayOrderItem::update([
                    'can_notify_account'=>1,
                    'can_notify_account_time'=>date('Y-m-d H:i:s')
                ], ['order_item_id'=>$r['order_item_id']]);
            }
        }
    }

    /**
     * @param $payment
     * @return false|float
     * 获取微信手续费
     */
    private function make_tip_clearing($payer)
    {
        $account = $payer->clearChannelTips($this->pay_order);
        if(empty($account)) {
            return 0;
        }

        $account['account_type'] = 4; //三方手续费 + 平台总帐
        $account['pay_type'] = 2; //支付
        $account['pay_sub_type'] = 406; //第三方渠道手续费

        $this->clear_list[] = [
            'pay_order_clearing_id' => array_shift($this->gen_ids),
            'pay_order_id' => $this->pay_order['pay_order_id'],
            'order_id' => $this->pay_order['order_id'],
            'account_id' => $account['account_id'],
            'account_type' => $account['account_type'],
            'pay_amount' => $account['amount'],
            'pay_type' => $account['pay_type'],
            'pay_sub_type' => $account['pay_sub_type'],
        ];

        return $account['amount'];
    }

    /**
     * @param $total
     * @return false|float
     * 计算平台收入
     */
    private function make_platform_clearing($payer)
    {
        $account = $payer->clearPlatformTips($this->pay_order);
        if(empty($account)) {
            return 0;
        }

        $account['account_type'] = 3; //平台（平台收入+平台补贴）
        $account['pay_type'] = 2; //支付
        $account['pay_sub_type'] = 405; //平台抽成

        $total_amount = $this->pay_order['pay_amount']; //订单支付总价格
        $cleared_amount = 0; //已经结算记录
        $cleared_tip = 0; //已经结算记录

        $pay_order_clearing_id = array_shift($this->gen_ids);
        $this->clear_list[] = [
            'pay_order_clearing_id' => $pay_order_clearing_id,
            'pay_order_id' => $this->pay_order['pay_order_id'],
            'order_id' => $this->pay_order['order_id'],
            'account_id' => $account['account_id'],
            'account_type' => 3,
            'pay_amount' => $account['amount'],
            'pay_type' => 2,
            'pay_sub_type' => 405,
        ];

        foreach ($this->order_items as $r) {

            $current_tip = Strategy::getTip($r['payment'], $total_amount, $cleared_amount, $account['amount'], $cleared_tip);
            $this->clear_items_list[] = [
                'pay_order_clearing_item_id' => array_shift($this->gen_ids),
                'pay_order_clearing_id' => $pay_order_clearing_id,
                'pay_order_id' => $this->pay_order['pay_order_id'],
                'order_id' => $this->pay_order['order_id'],
                'order_item_id' => $r['order_item_id'],
                'account_id' => $account['account_id'],
                'account_type' => 3,
                'pay_amount' => $current_tip,
                'clear_payment' => 0, //商品结算价格
                'pay_type' => 2,
                'pay_sub_type' => 405,
                'remark' => '',
            ];

            $cleared_amount += $r['payment'];
            $cleared_tip += $current_tip;
        }

        return $account;
    }

    /**
     * @return mixed
     * 计算分销信息, 分销没有总帐，仅有单条floor的计算
     */
    private function make_distribution_clearing()
    {
        /**
         * 'distributor_user_id'=>122234,
         * 'distributor_commission_value'=>60,
         * 'parent_user_id'=>122234 . rand(111, 999),
         * 'parent_commission_value'=>60,
         */
        $total_tip = 0;
        $maps = [];
        foreach ($this->order_items as $r) {
            //仅处理分销业务
            if(empty($r['distributor_user_id']) || $r['marketing_type'] != 1) {
                continue;
            }
            $r['distributor_commission_value'] = intval($r['distributor_commission_value']);
            $r['parent_commission_value'] = intval($r['parent_commission_value']);

            if($r['commission_mode'] == 1) { //按比例结算
                $_one = floor($r['distributor_commission_value'] * $this->order_info['total_price'] / 10000);
                $_two = floor($r['parent_commission_value'] * $this->order_info['total_price'] / 10000);

            } elseif($r['commission_mode'] == 2) {//固定金额结算
                $_one = $r['distributor_commission_value'];
                $_two = $r['parent_commission_value'];
            } else { //不处理
                continue;
            }

            $users[0] = [
                'user_id'=>$r['distributor_user_id'],
                'value'=>$r['distributor_commission_value'],
                'amount'=>$_one
            ];

            if(!empty($r['parent_user_id'])) {
                $users[1] = [
                    'user_id'=>$r['parent_user_id'],
                    'value'=>$r['parent_commission_value'],
                    'amount'=>$_two
                ];
            }

            foreach ($users as $u) {
                if (empty($u['user_id']) || $u['amount'] == 0) {
                    continue;
                }

                if(!isset($maps[$u['user_id']])) {
                    $maps[$u['user_id']] = [
                        'pay_order_clearing_id'=>array_shift($this->gen_ids),
                        'pay_order_id' => $this->pay_order['pay_order_id'],
                        'order_id' => $this->pay_order['order_id'],
                        'account_id' => $u['user_id'],
                        'account_type' => 1,
                        'pay_amount' => 0,
                        'pay_type' => 1,
                        'pay_sub_type' => 404,
                    ];
                }
                $maps[$u['user_id']]['pay_amount'] += $u['amount'];

                $this->clear_items_list[] = [
                    'pay_order_clearing_item_id' => array_shift($this->gen_ids),
                    'pay_order_clearing_id' => $maps[$u['user_id']]['pay_order_clearing_id'],
                    'pay_order_id' => $this->pay_order['pay_order_id'],
                    'order_id' => $this->pay_order['order_id'],
                    'order_item_id' => $r['order_item_id'],
                    'account_id' => $u['user_id'],
                    'account_type' => 1, //用户
                    'pay_amount' => $u['amount'],
                    'clear_payment' => 0, //商品结算价格
                    'pay_type' => 1, //收入
                    'pay_sub_type' => 404, //团长佣金收入
                    'remark' => '',
                ];
                $total_tip += $u['amount'];
            }
        }

        if($maps) {
            array_push($this->clear_list, array_values($maps));
        }

        return $total_tip;
    }

    /**
     * @param $merchant_cash
     * @return mixed
     * 计算商户清分信息, 存在结算价则需要二次平台收入
     */
    private function make_merchant_clearing($merchant_cash, $platform)
    {
        $m = $this->get_merchant_account($this->pay_order);

        $merchant['account_id'] = $m['account_id']; //商户
        $merchant['account_type'] = 2; //商户
        $merchant['pay_type'] = 1; //收入
        $merchant['pay_sub_type'] = 407; //商品交易102
        $merchant['amount'] = 0;

        $platform['account_type'] = 3;
        $platform['pay_type'] = 1;
        $platform['pay_sub_type'] = 408; //平台抽成（结算价）;
        $platform['amount'] = 0;

        $m_clearing_id = array_shift($this->gen_ids); //商家清分ID
        $p_clearing_id = array_shift($this->gen_ids); //平台结算价清分ID

        $total_amount = $this->pay_order['pay_amount']; //订单支付总价格
        $cleared_amount = 0; //已经结算金额记录
        $cleared_tip = 0; //已经结算tip记录
        $total_tip = $merchant_cash; //已经结算记录
        foreach ($this->order_items as $r) {

            $current_tip = Strategy::getTip($r['payment'], $total_amount, $cleared_amount, $total_tip, $cleared_tip);

            //商家资金需要显示商品名称
            $remark = $r['goods_name'] ?? '';
            if(mb_strlen($remark) > 100) {
                $remark = mb_substr($remark, 0, 100) . '...';
            }

            if(!empty($r['clear_price'])) {
                $clear_payment = $r['goods_num'] * $r['clear_price'];
                $platform_amount = $current_tip - $clear_payment;

            if(!empty($r['clear_price'])) {
                $clear_payment = $r['goods_num'] * $r['clear_price'];
            } else {
                $clear_payment = $current_tip;
                $platform_amount = 0;
            }

            if($platform_amount < 0) {
                throw new PayException(['cus'=>12]); //结算价不能大于商家分成部分
            }

            if($platform_amount > 0) {
                //结算价剩余部分返回平台
                $platform['amount'] += $platform_amount;
                $this->clear_items_list[] = [
                    'pay_order_clearing_item_id' => array_shift($this->gen_ids),
                    'pay_order_clearing_id' => $p_clearing_id,
                    'pay_order_id' => $this->pay_order['pay_order_id'],
                    'order_id' => $this->pay_order['order_id'],
                    'order_item_id' => $r['order_item_id'],
                    'account_id' => $platform['account_id'],
                    'account_type' => $platform['account_type'],
                    'pay_amount' => $platform_amount,
                    'clear_payment' => 0, //商品结算价格
                    'pay_type' => $platform['pay_type'],
                    'pay_sub_type' => $platform['pay_sub_type'],
                    'remark' => '',
                ];

                $pay_amount = $clear_payment;

            } else{}

            $this->clear_items_list[] = [
                'pay_order_clearing_item_id' => array_shift($this->gen_ids),
                'pay_order_clearing_id' => $m_clearing_id,
                'pay_order_id' => $this->pay_order['pay_order_id'],
                'order_id' => $this->pay_order['order_id'],
                'order_item_id' => $r['order_item_id'],
                'account_id' => $merchant['account_id'],
                'account_type' => $merchant['account_type'],
                'pay_amount' => $clear_payment, //商品结算价格
                'clear_payment' => 0,
                'pay_type' => $merchant['pay_type'],
                'pay_sub_type' => $merchant['pay_sub_type'],
                'remark' => $remark,
            ];

            $cleared_amount += $r['payment'];
            $cleared_tip += $current_tip;
        }

        //商户收益
        $this->clear_list[] = [
            'pay_order_clearing_id' => $m_clearing_id,
            'pay_order_id' => $this->pay_order['pay_order_id'],
            'order_id' => $this->pay_order['order_id'],
            'account_id' => $merchant['account_id'],
            'account_type' => $merchant['account_type'],
            'pay_amount' => $merchant['amount'],
            'pay_type' => $merchant['pay_type'],
            'pay_sub_type' => $merchant['pay_sub_type'],
        ];

        //结算价格收益返回平台
        if($platform['amount'] > 0) {
            $this->clear_list[] = [
                'pay_order_clearing_id' => $p_clearing_id,
                'pay_order_id' => $this->pay_order['pay_order_id'],
                'order_id' => $this->pay_order['order_id'],
                'account_id' => $platform['account_id'],
                'account_type' => $platform['account_type'],
                'pay_amount' => $platform['amount'],
                'pay_type' => $platform['pay_type'],
                'pay_sub_type' => $platform['pay_sub_type'],
            ];
        }

        return $merchant_cash;
    }

    private function get_idgen_id($number, $count = 1)
    {
        $number = substr($number, -2);
        $number = intval($number);
        $res = Idgen::get(appConfig('idgen.partner'), appConfig('idgen.key'), [], [[
            "type" => "55c768",
            'number' => $number,
            "count" => $count]]);
        $this->gen_ids = $res['id_datetime']['55c768'] ?? [];
        return $this->gen_ids;
    }

    private function get_merchant_account($params)
    {
        /*
        ///account/account/get_account
        $url = config('interface', 'merchant.lifeaccount.get_life_account_by_id');
        if (!$url) {
            throw new CodeSpecialException("failed" . __METHOD__);
        }
        */
        $url = "http://account.dev.yidian-inc.com/account/account/get_account";
        $ret = Sdk::call($url, [
            'life_account_id'=>$params['life_account_id'],
            'shop_id'=>$params['shop_id'],
            'source_name'=>$params['source_name'],
            'service_name'=>$params['service_name'],
        ]);
        if(empty($ret['result'])) {
            throw new PayException(['cus'=>9]);
        }
        return $ret['result'];
    }

    private function getNextTime($times = 0) {
        return time() + pow(3, $times + 1);
    }
}