<?php


namespace App\Services\refund;

use Api\PhpServices\Idgen\Idgen;
use App\Exception\BaseException;
use App\Models\order\mysql\PayOrder;
use App\Models\order\mysql\PayOrderItem;
use App\Models\order\mysql\PayCouponItem;
use App\Models\order\mysql\PayOrderClearing;
use App\Models\order\mysql\PayOrderClearingItem;
use App\Models\order\mysql\RefundMerchantRecord;
use App\Models\order\mysql\RefundOrder;
use App\Models\order\mysql\RefundOrderItem;

use App\Models\order\mysql\RefundPlatformRecord;
use App\Services\Channel;
use App\Services\marketing\AccountService;

use App\Services\pay\FundsService;
use Helpers\Strategy;

use App\Exception\custom\RefundException;
use Yaf\Application;

class RefundService
{

    /**
     * @param $order_id
     * @param array $order_item_ids
     * @return array
     * @throws BaseException
     * 发起退款请求
     */
    public function do_refund($order_id, $order_item_ids = []) {

        if(empty($order_id)) {
            throw new RefundException(['cus' => 1]);
        }

        $data = $this->can_refund($order_id, $order_item_ids);

        $mata_data = [
            'app_id'=>'merchant-c',
            'environ'=>Application::app()->environ(),
            'refund_order_id'=>$data['refund_order']['refund_order_id'],
            'order_item_id'=>$data['order_item_id'],
            'order_id'=>$order_id,
        ];

        try{
            $payer = Channel::getChannel($data['pay_order']['pay_channel']);
            $ret = $payer->refund($data, $mata_data);
            if(!empty($ret["data"])) {
                $edit = ['request_pingxx_success_time'=>date('Y-m-d H:i:s')];
                RefundOrder::update($edit, ['refund_order_id'=>$data['refund_order']['refund_order_id']]);
            } else {
                throw new BaseException(['msg'=>$ret['error']['message'], 'code'=>'2011']);
            }

            return [
                'order_item_id'=>$data['order_item_id'],
                'refund_order_id'=>$data['refund_order']['refund_order_id'],
            ];

        }catch (\Exception $e) {
            $edit = ['refund_order_status'=>0];
            RefundOrder::update($edit, ['refund_order_id'=>$data['refund_order']['refund_order_id']]);
            PayOrderItem::update(['refund_order_status'=>0], ['order_item_id'=>$data['order_item_id']]);

            throw $e;
        }
    }

    /**
     * @param $data
     * @return array
     * @throws RefundException
     * 退款回调
     */
    public function call_back($data) {

        if (!in_array($data['type'], ['refund.succeeded'])) {
            return ['error'=>'退款回调类型不正确'];
        }

        if (empty($data['data']['object'])) {
            return ['error'=>'退款回调格式不正确'];
        }

        $object = $data['data']['object'];
        if(empty($object['metadata']['app_id']) || $object['metadata']['app_id'] != "merchant-c") {
            //非支付中心发出的订单，需要忽略
            return ['error'=>'非merchant渠道订单，不做处理。'];
        }

        if($object['succeed'] != true) {
            return ['error'=>'退款状态不正确，不做处理。'];
        }

        if(empty($object['metadata']['refund_order_id'])) {
            return ['error'=>'退款订单缺少matadata，不做处理。'];
        }

        if(empty($object['metadata']['order_item_id'])) {
            return ['error'=>'退款订单缺少order_item_id，不做处理。'];
        }

        /**
        平台出账、手续费返还。
        营销补贴返还
        团长收入减少
        平台收入减少
        商户收入减少
         */

        // 修改状态，锁定业务
        $refund_order_id = $object['metadata']['refund_order_id'];

        RefundOrder::beginTransaction();
        $refund = RefundOrder::getMaster('*', ['refund_order_id'=>$refund_order_id]);

        $cnt = true;
        //营销退款处理
        $coupons = PayCouponItem::selectMaster('*',
            ['order_item_id'=>$object['metadata']['order_item_id']]);
        if($coupons) {
            $cnt = $cnt && AccountService::refund($coupons, $refund_order_id);
        }

        //总账资金记账
        $cnt = $cnt && FundsService::payOut([
                'trade_id'=>$refund['refund_order_id'],
                'third_order_id'=>$object['id'],
                'amount'=>$refund['refund_amount'],
                'tip'=>$refund['refund_wx_tip'],
            ], 1);


        $edit = [
            'refund_order_status'=>2, //pingxx回调成功,
            'pingxx_callback_success_time'=>date('Y-m-d H:i:s'), //回调成功时间
        ];

        $cnt = $cnt && RefundOrder::update($edit, ['refund_order_id'=>$refund_order_id, 'refund_order_status'=>1]);

        $cnt = $cnt && PayOrderItem::update(['refund_order_status'=>2], [
            'order_item_id'=>$object['metadata']['order_item_id']
        ]);

        $cnt = $cnt && $this->refund_clear_record($refund, $object['metadata']['order_item_id']);

        if($cnt) {
            RefundOrder::commit();
            return ['refund_order_id'=>$refund_order_id];
        } else {
            RefundOrder::rollback();
            throw new RefundException(['cus' => 16]);
        }
    }

    /**
     * @param $refund
     * @param array $order_item_ids
     * @return bool
     */
    private function refund_clear_record($refund, $order_item_ids = []) {

        $clear_items = PayOrderClearingItem::selectMaster('*',
            ['order_item_id'=>$order_item_ids]);

        $ids = $this->get_idgen_id($refund['user_id'], count($clear_items) * 4);

        $d = $m = $p = [];
        foreach ($clear_items as $r) {
            if($r['pay_sub_type'] == 404) { // 团长佣金收入
                if(!isset($d[$r['account_id']])) {
                    $d[$r['account_id']] = [
                        'refund_personal_id'=>array_shift($ids),
                        'refund_order_id'=>$refund['refund_order_id'],
                        'pay_order_id'=>$r['pay_order_id'],
                        'order_id'=>$r['order_id'],
                        'user_id'=>$r['account_id'],
                        'payment'=>0,
                    ];
                }
                $d[$r['account_id']]['payment'] += $r['pay_amount'];

            } elseif ($r['pay_sub_type'] == 405 || $r['pay_sub_type'] == 408) { // 平台抽成
                if(!isset($p[$r['account_id']])) {
                    $p[$r['account_id']] = [
                        'refund_platform_id'=>array_shift($ids),
                        'refund_order_id'=>$refund['refund_order_id'],
                        'pay_order_id'=>$r['pay_order_id'],
                        'order_id'=>$r['order_id'],
                        'account_id'=>$r['account_id'],
                        'payment'=>0,
                    ];
                }
                $p[$r['account_id']]['payment'] += $r['pay_amount'];

            } elseif ($r['pay_sub_type'] == 407) { // 商品交易
                if(!isset($m[$r['account_id']])) {
                    $m[$r['account_id']] = [
                        'refund_merchant_id'=>array_shift($ids),
                        'refund_order_id'=>$refund['refund_order_id'],
                        'pay_order_id'=>$r['pay_order_id'],
                        'order_id'=>$r['order_id'],
                        'account_id'=>$r['account_id'],
                        'payment'=>0,
                    ];
                }
                $m[$r['account_id']]['payment'] += $r['pay_amount'];


            } else {}
        }

        $cnt = true;
        $cnt = $cnt && RefundMerchantRecord::insert(array_values($m), ['rowCount' => true]);
        if($p) {
            $cnt = $cnt && RefundPlatformRecord::insert(array_values($p), ['rowCount' => true]);
        }
        if($d) {
            $cnt = $cnt && RefundPlatformRecord::insert(array_values($d), ['rowCount' => true]);
        }

        return $cnt;
    }

    /**
     * @param $order_id
     * @param $order_item_ids
     * @return array
     * @throws BaseException
     * 判断订单是否可退，判断是否存在退款申请
     */
    private function can_refund($order_id, $order_item_ids = []) {

        /**
         * 业务方自行解决，订单关闭、订单超过时长不允许退款等逻辑
         * 支付中心退款判断仅做主单是否支付，主单是否支持退款的判断
         */

        $refund_payment = 0;
        try{
            RefundOrder::beginTransaction();
            $pay_order = PayOrder::getMaster('*', ['order_id'=>$order_id]);
            if(empty($pay_order) || $pay_order['pay_order_status'] != 2) {
                throw new RefundException(['cus' => 2]);
            }

            $pay_order_items = PayOrderItem::selectMaster('*', ['pay_order_id'=>$pay_order['pay_order_id']]);
            foreach ($pay_order_items as $item) {
                if($item['can_notify_account'] == 1) {
                    continue; //已结算不允许退款
                }

                if(in_array($item['refund_order_status'], [1, 2])) {
                    continue; //退款中、已退款不允许再次申请
                }

                if(!empty($order_item_ids) && !in_array($item['order_item_id'], $order_item_ids)) {
                    continue; //部分退款，且不再退款子单列表，不处理
                }

                $items[] = $item;
                $refund_payment += $item['payment'];
            }

            if(empty($items)) {
                throw new RefundException(['cus' => 3]);
            }

            //重新计算微信手续费
            $cleared_amount = $cleared_tip = 0;

            // 计算已退金额、已退手续费 sum(refund_wx_tip), sum(refund_amount)
            $list = RefundOrder::selectMaster('*', ['order_id'=>$pay_order['order_id']]);
            foreach ($list as $r) {
                if($r['refund_order_status'] == 0) {
                    continue;
                }

                $cleared_amount += $r['refund_amount'];
                $cleared_tip += $r['refund_wx_tip'];
            }

            // 总共手续费
            $wx_clear = PayOrderClearing::getMaster('*', [
                    'order_id'=>$pay_order['order_id'],
                    'pay_order_id'=>$pay_order['pay_order_id'],
                    'pay_sub_type'=>406 //三方渠道手续费
                ]
            );
            $wx_total_tip = 0;
            if($wx_clear) {
                $wx_total_tip = $wx_clear['pay_amount'];
            }

            $wx_tip = Strategy::getWxTip($refund_payment, $pay_order['pay_amount'], $cleared_amount, $wx_total_tip, $cleared_tip);

            $ids = $this->get_idgen_id($pay_order['user_id'], count($items) + 1);
            $refund_order_id = array_shift($ids);
            $refund = [
                'refund_order_id'=>$refund_order_id,
                'user_id'=>$pay_order['user_id'],
                'order_id'=>$pay_order['order_id'],
                'refund_order_status'=>1,
                'refund_amount'=>$refund_payment,
                'refund_wx_tip'=>$wx_tip,
                'source_name'=>$pay_order['source_name'],
                'extra'=> json_encode(['order_item_id'=>array_column($items, 'order_item_id')]),
            ];

            $refund_items = [];
            foreach ($items as $r) {
                $refund_items[] = [
                    'refund_order_item_id'=>array_shift($ids),
                    'refund_order_id'=>$refund_order_id,
                    'order_item_id'=>$r['order_item_id'],
                ];
            }

            $cnt = true;
            $cnt = $cnt && RefundOrder::insert($refund, ['rowCount'=>true]);
            $cnt = $cnt && RefundOrderItem::insert($refund_items, ['rowCount'=>true]);
            $cnt = $cnt && PayOrderItem::update(['refund_order_status'=>1], ['order_item_id'=>array_column($items, 'order_item_id')]);

            if($cnt == false) {
                throw new RefundException(['cus' => 4]);
            }

            RefundOrder::commit();

        }catch (\Exception $e) {
            RefundOrder::rollback();
            throw new BaseException(['msg'=>$e->getMessage(), 'code'=>$e->getCode()]);
        }

        return [
            'refund_order'=>$refund,
            'pay_order'=>$pay_order,
            'order_item_id'=>array_column($items, 'order_item_id'),
        ];
    }

    private function get_idgen_id($number, $count = 1)
    {
        $number = substr($number, -2);
        $number = intval($number);
        $res = Idgen::get(appConfig('idgen.partner'), appConfig('idgen.key'), [], [[
            "type" => "55c768",
            'number' => $number,
            "count" => $count]]);
        return $res['id_datetime']['55c768'] ?? [];
    }

}